@@ -2,14 +2,18 @@
 
 import logging
 import datetime
+from multiprocessing.managers import BaseManager, DictProxy
 import string
 import sys
 import os
 import traceback
 import socket
-from fireworks.core.fw_config import FWConfig
+import multiprocessing
 
-__author__ = 'Anubhav Jain'
+from fireworks.core.fw_config import FWConfig, FWData
+
+
+__author__ = 'Anubhav Jain, Xiaohui Qu'
 __copyright__ = 'Copyright 2012, The Materials Project'
 __version__ = '0.1'
 __maintainer__ = 'Anubhav Jain'
@@ -38,7 +42,6 @@ def get_fw_logger(name, l_dir=None, file_levels=('DEBUG', 'ERROR'), stream_level
     logger.setLevel(logging.DEBUG)  # anything debug and above passes through to the handler level
 
     stream_level = stream_level if stream_level else 'CRITICAL'
-
     # add handlers for the file_levels
     if l_dir:
         for lvl in file_levels:
@@ -62,17 +65,30 @@ def get_fw_logger(name, l_dir=None, file_levels=('DEBUG', 'ERROR'), stream_level
     return logger
 
 
-def log_fancy(m_logger, log_lvl, msgs, add_traceback=False):
+def log_multi(m_logger, msg, log_lvl='info'):
+    """
+    :param m_logger: (logger) The logger object
+    :param msg: (str) a String to log
+    :param log_lvl: (str) The level to log at
+    """
+    _log_fnc = getattr(m_logger, log_lvl.lower())
+    if FWData().MULTIPROCESSING:
+        _log_fnc("{} : ({})".format(msg, multiprocessing.current_process().name))
+    else:
+        _log_fnc(msg)
+
+
+def log_fancy(m_logger, msgs, log_lvl='info', add_traceback=False):
     """
     A wrapper around the logger messages useful for multi-line logs.
     Helps to group log messages by adding a fancy border around it,
     which enhances readability of log lines meant to be read
     as a unit.
     
-    :param m_logger: The logger object
-    :param log_lvl: The level to log at
-    :param msgs: a String or iterable of Strings
-    :param add_traceback: add traceback text, useful when logging exceptions (default False)
+    :param m_logger: (logger) The logger object
+    :param log_lvl: (str) The level to log at
+    :param msgs: ([str]) a String or iterable of Strings
+    :param add_traceback: (bool) add traceback text, useful when logging exceptions (default False)
     """
 
     if isinstance(msgs, basestring):
@@ -91,10 +107,10 @@ def log_exception(m_logger, msgs):
     """
     A shortcut wrapper around log_fancy for exceptions
     
-    :param m_logger: The logger object
-    :param msgs: An iterable of Strings, will be joined by newlines
+    :param m_logger: (logger) The logger object
+    :param msgs: ([str]) String or iterable of Strings, will be joined by newlines
     """
-    return log_fancy(m_logger, 'error', msgs, add_traceback=True)
+    return log_fancy(m_logger, msgs, 'error', add_traceback=True)
 
 
 def create_datestamp_dir(root_dir, l_logger, prefix='block_'):
@@ -114,6 +130,8 @@ def create_datestamp_dir(root_dir, l_logger, prefix='block_'):
     l_logger.info('Created new dir {}'.format(full_path))
     return full_path
 
+
+
 def get_my_ip():
     try:
         return socket.gethostbyname(socket.gethostname())
@@ -128,4 +146,45 @@ def get_my_host():
 def get_slug(m_str):
     valid_chars = "-_.() %s%s" % (string.ascii_letters, string.digits)
     m_str = ''.join(c for c in m_str if c in valid_chars)
-    return m_str.replace(' ', '_')
\ No newline at end of file
+    return m_str.replace(' ', '_')
+
+
+def acquire_db_lock():
+    """
+    Acquire lock on database
+    """
+    if FWData().MULTIPROCESSING:
+        FWData().PROCESS_LOCK.acquire()
+
+
+def release_db_lock(safe=True):
+    """
+    Release lock on database
+    :param safe: (bool) ignore errors (e.g., lock never acquired)
+    """
+    if FWData().MULTIPROCESSING:
+        try:
+            FWData().PROCESS_LOCK.release()
+        except ValueError, ve:
+            if safe:
+                raise ValueError(ve)
+
+
+class DataServer(BaseManager):
+    """
+    Provide a server that can host shared objects between multiprocessing
+    Processes (that normally can't share data). For example, a common LaunchPad is
+    shared between processes and pinging launches is coordinated to limit DB hits.
+    """
+
+    @classmethod
+    def setup(cls, launchpad):
+        """
+        :param launchpad: (LaunchPad) object
+        :return:
+        """
+        DataServer.register('LaunchPad', callable=lambda: launchpad)
+        DataServer.register('Running_IDs', callable=lambda: {}, proxytype=DictProxy)
+        m = DataServer(address=('127.0.0.1', 0), authkey=FWConfig().DS_PASSWORD)  # random port
+        m.start()
+        return m
\ No newline at end of file
